@@ -26,8 +26,10 @@
 #include "valueschangedcommand.h"
 
 #include "sharedmemory.h"
+
 #include <QCache>
 #include <QDebug>
+#include <QIODevice>
 
 #include <cstring>
 
@@ -50,7 +52,7 @@ ValuesChangedCommand::ValuesChangedCommand(const QVector<PropertyValueContainer>
 {
 }
 
-QVector<PropertyValueContainer> ValuesChangedCommand::valueChanges() const
+const QVector<PropertyValueContainer> ValuesChangedCommand::valueChanges() const
 {
     return m_valueChangeVector;
 }
@@ -62,7 +64,7 @@ quint32 ValuesChangedCommand::keyNumber() const
 
 void ValuesChangedCommand::removeSharedMemorys(const QVector<qint32> &keyNumberVector)
 {
-    foreach (qint32 keyNumber, keyNumberVector) {
+    for (qint32 keyNumber : keyNumberVector) {
         SharedMemory *sharedMemory = globalSharedMemoryContainer()->take(keyNumber);
         delete sharedMemory;
     }
@@ -95,15 +97,26 @@ QDataStream &operator<<(QDataStream &out, const ValuesChangedCommand &command)
 {
     static const bool dontUseSharedMemory = qEnvironmentVariableIsSet("DESIGNER_DONT_USE_SHARED_MEMORY");
 
-    if (!dontUseSharedMemory && command.valueChanges().count() > 5) {
+    QVector<PropertyValueContainer> propertyValueContainer = command.valueChanges();
+
+    if (command.transactionOption != ValuesChangedCommand::TransactionOption::None) {
+        PropertyValueContainer optionContainer(command.transactionOption);
+        propertyValueContainer.append(optionContainer);
+    }
+
+    if (!dontUseSharedMemory && propertyValueContainer.count() > 5) {
         static quint32 keyCounter = 0;
         ++keyCounter;
         command.m_keyNumber = keyCounter;
         QByteArray outDataStreamByteArray;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         QDataStream temporaryOutDataStream(&outDataStreamByteArray, QIODevice::WriteOnly);
+#else
+        QDataStream temporaryOutDataStream(&outDataStreamByteArray, QDataStream::WriteOnly);
+#endif
         temporaryOutDataStream.setVersion(QDataStream::Qt_4_8);
 
-        temporaryOutDataStream << command.valueChanges();
+        temporaryOutDataStream << propertyValueContainer;
 
         SharedMemory *sharedMemory = createSharedMemory(keyCounter, outDataStreamByteArray.size());
 
@@ -118,7 +131,7 @@ QDataStream &operator<<(QDataStream &out, const ValuesChangedCommand &command)
     }
 
     out << qint32(0);
-    out << command.valueChanges();
+    out << propertyValueContainer;
 
     return out;
 }
@@ -144,17 +157,29 @@ QDataStream &operator>>(QDataStream &in, ValuesChangedCommand &command)
 {
     in >> command.m_keyNumber;
 
-    if (command.keyNumber() > 0) {
-        readSharedMemory(command.keyNumber(), &command.m_valueChangeVector);
-    } else {
-        in >> command.m_valueChangeVector;
+    QVector<PropertyValueContainer> valueChangeVector;
+
+    if (command.keyNumber() > 0)
+        readSharedMemory(command.keyNumber(), &valueChangeVector);
+    else
+        in >> valueChangeVector;
+
+    // '-option-' is not a valid property name and indicates that we store the transaction option.
+    if (!valueChangeVector.isEmpty() && valueChangeVector.last().name() == "-option-") {
+        command.transactionOption =
+            static_cast<ValuesChangedCommand::TransactionOption>(valueChangeVector.last().instanceId());
+        valueChangeVector.removeLast();
     }
+
+    command.m_valueChangeVector = valueChangeVector;
+
     return in;
 }
 
 bool operator ==(const ValuesChangedCommand &first, const ValuesChangedCommand &second)
 {
-    return first.m_valueChangeVector == second.m_valueChangeVector;
+    return first.m_valueChangeVector == second.m_valueChangeVector
+        && first.transactionOption == second.transactionOption;
 }
 
 QDebug operator <<(QDebug debug, const ValuesChangedCommand &command)
@@ -164,4 +189,15 @@ QDebug operator <<(QDebug debug, const ValuesChangedCommand &command)
                     << command.valueChanges() << ")";
 }
 
+QDataStream &operator<<(QDataStream &out, const ValuesModifiedCommand &command)
+{
+    return out << static_cast<const ValuesChangedCommand &>(command);
+}
+
+QDataStream &operator>>(QDataStream &in, ValuesModifiedCommand &command)
+{
+    return in >> static_cast<ValuesChangedCommand &>(command);
+}
+
+
 } // namespace QmlDesigner
